﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq.Expressions;

namespace AutocompleteBox
{
	public class AutocompleteModel : INotifyPropertyChanged
	{
		private string text;
		private string selectedItem;
		private bool isDropDownOpen;
		private IAutocompleteProvider autocompleteProvider;

		public AutocompleteModel()
		{
			this.Suggestions = new ObservableCollection<string>();
		}

		public event PropertyChangedEventHandler PropertyChanged;

		public int MinimumTextLength { get; set; }

		public string Text
		{
			get
			{
				return this.text;
			}
			set
			{
				if (this.text != value)
				{
					this.text = value;
					this.OnPropertyChanged(() => this.Text);

					if (this.text != this.selectedItem)
					{
						this.IsDropDownOpen = true;
					}

					if (value.Length >= this.MinimumTextLength || value.Length == 0)
					{
						this.AutocompleteProvider.GetSuggestions(value);
					}
				}
			}
		}

		public string SelectedItem
		{
			get
			{
				return this.selectedItem;
			}
			set
			{
				if (this.selectedItem != value)
				{
					this.selectedItem = value;
					this.OnPropertyChanged(() => this.SelectedItem);
				}
			}
		}

		public bool IsDropDownOpen
		{
			get
			{
				return this.isDropDownOpen;
			}
			set
			{
				if (this.isDropDownOpen != value)
				{
					this.isDropDownOpen = value;
					this.OnPropertyChanged(() => this.IsDropDownOpen);
				}
			}
		}

		public ObservableCollection<string> Suggestions
		{
			get;
			private set;
		}

		public IAutocompleteProvider AutocompleteProvider
		{
			get
			{
				return this.autocompleteProvider;
			}
			set
			{
				if (this.autocompleteProvider != null)
				{
					this.autocompleteProvider.SuggestionsReceived -= this.OnSuggestionsReceived;
				}
				this.autocompleteProvider = value;
				if (this.autocompleteProvider != null)
				{
					this.autocompleteProvider.SuggestionsReceived += this.OnSuggestionsReceived;
				}
			}
		}

		protected virtual void OnPropertyChanged<T>(Expression<Func<T>> propertyExpression)
		{
			if (this.PropertyChanged != null)
			{
				MemberExpression memberExpression = (MemberExpression)propertyExpression.Body;

				this.PropertyChanged(this, new PropertyChangedEventArgs(memberExpression.Member.Name));
			}
		}

		private void OnSuggestionsReceived(object sender, SuggestionsReceivedEventArgs e)
		{
			bool emptySelection = string.IsNullOrEmpty(this.Text);

			for (int i = this.Suggestions.Count - 1; i >= 0; i--)
			{
				if (emptySelection || this.Suggestions[i] != this.SelectedItem)
				{
					this.Suggestions.RemoveAt(i);
				}
			}

			foreach (string word in e.Suggestions)
			{
				if (!this.Suggestions.Contains(word))
				{
					this.Suggestions.Insert(0, word);
				}
			}
		}
	}
}