﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Xml.Linq;

namespace AutocompleteBox
{
	public class GoogleAutocompleteProvider : IAutocompleteProvider
	{
#if SILVERLIGHT
		private const string AutocompleteUrl = "../search.ashx?client=ie8&q=";
#else
		private const string AutocompleteUrl = "http://clients5.google.com/complete/search?hl=en-us&client=ie8&q=";
#endif

		WebClient client;

		public GoogleAutocompleteProvider()
		{
			this.client = new WebClient();
		}

		public event SuggestionsReceivedEventHandler SuggestionsReceived;

		public void GetSuggestions(string text)
		{
			DownloadStringCompletedEventHandler handler = null;
			if (this.client.IsBusy)
			{
				handler = (s, e) =>
					{
						this.client.DownloadStringCompleted -= handler;
						if (e.Cancelled)
						{
							this.GetSuggestions(text);
						}
					};
				this.client.DownloadStringCompleted += handler;
				this.client.CancelAsync();
			}
			else
			{
				handler = (s, e) =>
				{
					this.client.DownloadStringCompleted -= handler;
					if (!e.Cancelled)
					{
						this.OnSuggestionsReceived(this.ParseResult(e.Result));
					}
				};
				this.client.DownloadStringCompleted += handler;
				this.client.DownloadStringAsync(new Uri(AutocompleteUrl + text, UriKind.RelativeOrAbsolute));
			}
		}

		private IList<string> ParseResult(string result)
		{
			var xml = XDocument.Parse((string)result);

			var values = new List<string>();
			foreach (var node in xml.Descendants(XName.Get("Text", "http://opensearch.org/searchsuggest2")))
			{
				values.Add(node.Value);
			}

			return values;
		}

		protected virtual void OnSuggestionsReceived(IList<string> results)
		{
			if (this.SuggestionsReceived != null)
			{
				this.SuggestionsReceived(this, new SuggestionsReceivedEventArgs(results));
			}
		}
	}
}